import datetime
import os
import re
from typing import Optional, Union, List

import pydantic
import ujson
from deta import Deta
from pydantic import Field, BaseModel, ValidationError

from odetam.field import DetaField
from odetam.query import DetaQuery, DetaQueryStatement, DetaQueryList


class DetaError(BaseException):
    pass


class ItemNotFound(DetaError):
    pass


class NoProjectKey(DetaError):
    pass


class InvalidDetaQuery(DetaError):
    pass


DETA_BASIC_TYPES = [dict, list, str, int, float, bool]
DETA_OPTIONAL_TYPES = [Optional[type_] for type_ in DETA_BASIC_TYPES]
DETA_BASIC_LIST_TYPES = [
    List[type_] for type_ in DETA_BASIC_TYPES + DETA_OPTIONAL_TYPES
]
DETA_TYPES = DETA_BASIC_TYPES + DETA_OPTIONAL_TYPES + DETA_BASIC_LIST_TYPES


class Alert:
    """Use this class to make alerts on any page"""
    def __init__(self, message = None, alert_type = 'danger'):
        self.type = alert_type
        self.message = message


def handle_db_property(cls, deta_class):
    if cls._db:
        return cls._db
    try:
        # changed to 'DETA_PROJECT_KEY' because Deta SDK searches for it by default
        # https://github.com/deta/deta-python/blob/master/deta/utils.py
        deta = deta_class(os.getenv("DETA_PROJECT_KEY"))
    except (AttributeError, AssertionError, ValueError):
        raise NoProjectKey(
            "Ensure that the 'DETA_PROJECT_KEY' environment variable is set to your "
            "project key and then restart the server."
        )
    cls._db = deta.Base(cls.__db_name__)
    return cls._db


class DetaModelMetaClass(pydantic.main.ModelMetaclass):
    def __new__(mcs, name, bases, dct):
        cls = super().__new__(mcs, name, bases, dct)
        cls.Config.arbitrary_types_allowed = True  # but, be careful if you use them!
        if getattr(cls.Config, "table_name", None):
            cls.__db_name__ = cls.Config.table_name
        else:
            cls.__db_name__ = re.sub(r"(?<!^)(?=[A-Z])", "_", name).lower()
        cls._db = None

        for name, field in cls.__fields__.items():
            setattr(cls, name, DetaField(field=field))
        return cls

    @property
    def __db__(cls):
        return handle_db_property(cls, Deta)


class BaseDetaModel(BaseModel):
    __db__ = None
    key: Optional[str] = Field(
        None, title="Key", description="Primary key in the database"
    )

    # overwriting odetam's implementation
    def _serialize(self, exclude=None):
        if not exclude:
            exclude = []
        as_dict = {}
        for field_name, field in self.__class__.__fields__.items():
            if field_name == "key" and not self.key:
                continue
            if field_name in exclude:
                continue
            # this originally failed when 0, 0.0, or False. Now we only check for None instances
            if getattr(self, field_name, None) is None:
                as_dict[field_name] = None
                continue
            if field.type_ in DETA_TYPES:
                as_dict[field_name] = getattr(self, field_name)
            elif field.type_ == datetime.datetime:
                as_dict[field_name] = getattr(self, field_name).timestamp()
            elif field.type_ == datetime.date:
                as_dict[field_name] = int(getattr(self, field_name).strftime("%Y%m%d"))
            elif field.type_ == datetime.time:
                as_dict[field_name] = int(
                    getattr(self, field_name).strftime("%H%M%S%f")
                )
            else:
                as_dict[field_name] = ujson.loads(self.json(include={field_name}))[
                    field_name
                ]
        return as_dict

    # overwriting odetam's implementation
    @classmethod
    def _deserialize(cls, data):
        as_dict = {}
        for field_name, field in cls.__fields__.items():
            # this originally failed when 0, 0.0, or False. Now we only check for None instances
            if field_name not in data:
                as_dict[field_name] = field.type_()
            elif field.type_ in DETA_TYPES:
                as_dict[field_name] = data[field_name]
            elif field.type_ == datetime.datetime:
                as_dict[field_name] = datetime.datetime.fromtimestamp(data[field_name])
            elif field.type_ == datetime.date:
                as_dict[field_name] = datetime.datetime.strptime(
                    str(data[field_name]), "%Y%m%d"
                ).date()
            elif field.type_ == datetime.time:
                as_dict[field_name] = datetime.datetime.strptime(
                    str(data[field_name]), "%H%M%S%f"
                ).time()
            else:
                try:
                    as_dict[field_name] = ujson.loads(data.get(field_name))
                except (TypeError, ValueError):
                    as_dict[field_name] = data.get(field_name, None)
        return cls.parse_obj(as_dict)

    @classmethod
    def _return_item_or_raise(cls, item):
        if item is None or item.get("key") == "None":
            raise ItemNotFound("Could not find item matching that key")
        try:
            return cls._deserialize(item)
        except ValidationError:
            raise ItemNotFound("Could not find item matching that key")


class DetaModel(BaseDetaModel, metaclass=DetaModelMetaClass):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.key = self.key or None

    @classmethod
    def get(cls, key):
        """
        Get a single instance
        :param key: Deta database key
        :return: object found in database serialized into its pydantic object

        :raises ItemNotFound: No matching item was found
        """
        item = cls.__db__.get(key)
        return cls._return_item_or_raise(item)

    @classmethod
    def get_all(cls):
        """Get all the records from the database"""
        records = cls.__db__.fetch().items
        return [cls._deserialize(record) for record in records]

    @classmethod
    def query(
        cls, query_statement: Union[dict, DetaQuery, DetaQueryStatement, DetaQueryList]
    ):
        """Get items from database based on the query."""
        if any(isinstance(query_statement, d) 
                          for d in [DetaQuery, DetaQueryStatement, DetaQueryList]):
            query_statement = query_statement.as_query()
        found = cls.__db__.fetch(query_statement).items
        return [cls._deserialize(item) for item in found]

    @classmethod
    def delete_key(cls, key):
        """Delete an item based on the key"""
        cls.__db__.delete(key)

    @classmethod
    def put_many(cls, items):
        """Put multiple instances at once

        :param items: List of pydantic objects to put in the database
        :returns: List of items successfully added, serialized with pydantic
        """
        records = []
        processed = []
        for item in items:
            exclude = set()
            if item.key is None:
                exclude = {"key"}
            # noinspection PyProtectedMember
            records.append(item._serialize(exclude=exclude))
            if len(records) == 25:
                result = cls.__db__.put_many(records)
                processed.extend(result["processed"]["items"])
                records = []
        if records:
            result = cls.__db__.put_many(records)
            processed.extend(result["processed"]["items"])
        return [cls._deserialize(rec) for rec in processed]

    # overwriting odetam's 'save/put' implementation
    # to include extra arguments supported by Deta
    @classmethod
    def _db_put(cls, data, expire_in, expire_at):
        return cls.__db__.put(data, expire_in=expire_in, expire_at=expire_at)
    
    def save(self, expire_in: int or None = None, expire_at: int or None = None):
        """Saves the record to the database. Behaves as upsert, will create
        if not present. Database key will then be set on the object."""
        saved = self._db_put(self._serialize(), expire_in, expire_at)
        self.key = saved["key"]

    def update(self, data: dict):
        """Updates the record in the database with the provided data."""
        for k, v in data.items():
            if k in self.__dict__:
                setattr(self, k, v)
        self.save()

    def delete(self):
        """Delete the open object from the database. The object will still exist in
        python, but will be deleted from the database and the key attribute will be
        set to None."""
        if not self.key:
            raise DetaError("Item does not have key for deletion")
        self.delete_key(self.key)
        self.key = None

    def get_attribute_value(self, index=1):
        d = self.__dict__
        return d[list(d.keys())[index]]


from fastapi import APIRouter, Request
from fastapi.responses import RedirectResponse
from fastapi.templating import Jinja2Templates

from {obj}.model import {Obj}


{obj}_router = APIRouter()
templates = Jinja2Templates(directory="")


# INDEX
@{obj}_router.get('/')
def index(request: Request):
    {obj}_list = {Obj}.get_all()
    return templates.TemplateResponse(
        '{obj}/templates/index.html',
        context={{'request': request, '{obj}_list': {obj}_list }})


# CREATE
@{obj}_router.get('/new')
def new(request: Request):
    return templates.TemplateResponse(
        '{obj}/templates/form.html',
        context={{'request': request, 'vals': dict() }})


@{obj}_router.post('/new', response_model={Obj})
async def create(request: Request):
    form_data = await request.form()
    {obj} = {Obj}.parse_obj(form_data)
    {obj}.save()
    return RedirectResponse(f'/{obj}/{{{obj}.key}}', status_code=303)


# UPDATE
@{obj}_router.get('/edit/{{key}}')
def edit(request: Request, key: str):
    vals = {Obj}.get(key)
    return templates.TemplateResponse(
        '{obj}/templates/form.html',
        context={{'request': request, 'vals': vals.dict() }})


@{obj}_router.post("/edit/{{key}}")
async def update(request: Request, key: str):
    in_db = {Obj}.get(key)
    update_data = await request.form()
    in_db.update(update_data)
    return RedirectResponse(f'/{obj}/{{key}}', status_code=303)


# DELETE
@{obj}_router.get('/delete/{{key}}')
def delete(request: Request, key: str):
    {Obj}.delete_key(key)
    return RedirectResponse(f'/{obj}', status_code=303)


# VIEW
@{obj}_router.get('/{{key}}')
def view(request: Request, key: str):
    {obj} = {Obj}.get(key)
    return templates.TemplateResponse(
        '{obj}/templates/view.html',
        context={{'request': request, '{obj}': {obj} }})



from dotenv import load_dotenv
from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles

from static_pages.router import static_pages_router

load_dotenv()

app = FastAPI(docs_url=None, redoc_url=None)
app.mount("/static", StaticFiles(directory="static_pages/static"), name="static")

app.include_router(static_pages_router, tags=["pages"], prefix="")



import json 
from pathlib import Path
import os

CONFIG_PATH = Path(__file__).parent.resolve() / 'user_config.json'

STYLES = {'BOOTSTRAP', 'MKDOCS'}

def __fetch_config():
    with open(CONFIG_PATH, 'r') as j:
        return json.loads(j.read())

def set_project_key(project_key: str):
    conf = __fetch_config()
    conf['project_key'] = project_key 
    with open(CONFIG_PATH, 'w') as j:
        j.write(json.dumps(conf, indent=4))

def clear_project_key():
    set_project_key("")

def config(key: str or None = None):
    if key:
        return __fetch_config().get(key)
    return __fetch_config()

def valid_style(style: str):
    if style.upper() in STYLES:
        return style.upper()
    else:
        return None

def run_server():
    server_cmd = "uvicorn main:app --reload"
    if check_project_type() == "MKDOCS":
        server_cmd = "mkdocs serve"
    os.system(server_cmd)

def check_project_type():
    project_type = "BOOTSTRAP"
    for (_, _, filenames) in os.walk(os.curdir):
        for fn in filenames:
            if 'mkdocs.yml' in fn:
                project_type = "MKDOCS"
    return project_type



import typer
import os
from detamvc import __version__
from detamvc.generator import build_base, gen_scaffold, gen_authlib
from typing import List
import detamvc.utilities as utils

app = typer.Typer()


@app.command()
def version():
    """ show current version of DetaMVC"""
    typer.echo(f"DetaMVC {__version__}")

@app.command()
def new(project: str, style: str = "bootstrap"):
    """ create a new project """
    if utils.valid_style(style):
        new_folder = os.path.join(os.curdir, project)
        try:      
            typer.secho(f"\nBuilding new project: {project}\n", fg='green')
            build_base(new_folder, project, style.upper())
            typer.echo(f"\n{project} was created.\n")
        except FileExistsError:
            typer.secho(f"'{project}' already exists in this folder.\n", fg='red')
    else:
        typer.secho(
            f"'{style}' is not a valid style for a project. "
            "Valid options are: {utils.STYLES}", fg='red')

@app.command()
def server():
    """ run the app locally """
    utils.run_server()

@app.command()
def s():
    """ alias for 'server' """
    server()

@app.command()
def build():
    """ alias for mkdocs build """
    os.system("mkdocs build")

@app.command()
def scaffold(obj: str, attributes: List[str]):
    """ create a router and views for a described object """
    if utils.check_project_type() == "MKDOCS":
        typer.secho(f"Scaffolding is not currently supported for MKDOCS styled apps", fg='red')
    else:
        typer.secho(f"\nScaffolding views and router for: {obj}\n", fg='green')
        gen_scaffold(os.curdir, obj, attributes)
        typer.echo(f"\n{obj} was created.\n")

@app.command()
def auth():
    """ generate authorization framework for users """
    if utils.check_project_type() == "MKDOCS":
        typer.secho(f"User Auth is not currently supported for MKDOCS styled apps", fg='red')
    else:
        typer.secho(f"\nGenerating views and router for: User\n", fg='green')
        gen_authlib(os.curdir)
        typer.echo(f"\nAuth was created.\n")

@app.command()
def set_project_key(project_key: str):
    """ set your development project key for Deta for faster builds """
    utils.set_project_key(project_key)
    typer.secho("Project Key successfully set.", fg='green')

@app.command()
def clear_project_key():
    """ clear your Deta development project key """
    utils.clear_project_key()
    typer.secho("Project Key successfully cleared.", fg='green')

@app.command()
def view_config():
    """ view your development configurations """
    conf = utils.config()
    typer.echo(conf)



from fastapi import APIRouter, Request
from fastapi.templating import Jinja2Templates

static_pages_router = APIRouter()
templates = Jinja2Templates(directory="")

@static_pages_router.get('/')
def index(request: Request):
    return templates.TemplateResponse(
        'static_pages/templates/index.html',
        context={{'request': request }})


from dataclasses import dataclass
from pathlib import Path
from os import walk, makedirs
from secrets import token_urlsafe
from detamvc.utilities import config


"""
HELPERS
"""


def __builder(
    project_path: str, 
    generator_path:str, 
    format_attrs: dict, 
    scaffold_obj_name=''
) -> None:
    """Build a project scaffold from a generator template.
    
    Args:
        project_path (str): The path to the directory where the project scaffold should be created.  
        generator_path (str): The path to the directory containing the generator template files.  
        format_attrs (dict): A dictionary of attributes to be used in formatting the generator template files.  
        scaffold_obj_name (str, optional): The name of the object being scaffolded. Defaults to an empty string.  
    """
    project_path = Path(project_path)
    generator_path = Path(__file__).parent.resolve() / generator_path
    for (dirpath, _, filenames) in walk(generator_path):
        np = (Path(scaffold_obj_name) / project_path 
              if scaffold_obj_name else project_path)
        new_path = dirpath.replace(str(generator_path), str(np))
        if '__pycache__' in str(dirpath):
            continue
        makedirs(new_path, exist_ok=True)
        for filename in filenames:
            full_path = Path(dirpath) / filename
            print(f">> {Path(new_path) / filename}")
            with open(full_path, 'r') as c:
                content = c.read()          
            
            with open(Path(new_path) / filename, 'w+') as o:
                o.write(__format(content, format_attrs))


def __format(doc: str, attr_dict: dict) -> str:
    """Format a document with the specified attribute dictionary.
    
    Args:
        doc (str): The document to be formatted.  
        attr_dict (dict): A dictionary of attributes to be used in the formatting.  
        
    Returns:
        str: The formatted string.
    """
    built_attrs = {}
    for k, v in attr_dict.items():
        if f'{{{k}}}' in doc:
            built_attrs.update({k: v})
    doc = doc.format(**built_attrs)
    return doc


def __get_object_attrs(attributes: list) -> dict:
    """Get a dictionary of object attributes from a list of attribute strings.
    
    Args:
        attributes (list): A list of strings representing object attributes in the form "name:type".  
        
    Returns:
        dict: A dictionary of object attributes, 
            where the keys are the attribute names and the values are the attribute types.
    """
    obj_attrs = dict()
    for a in attributes:
        if not a:
            continue
        s = a.split(':')
        obj_attrs[s[0]] = 'str' if len(s) == 1 else s[1].lower()
    return obj_attrs

def __prepare_model_attrs(obj_attrs: dict) -> str:
    """Provides attributes for the DetaModel  

    Args:
        obj_attrs (dict): attributes dictionary  

    Returns:
        str: prepared attributes as string
    """
    model_attrs = [
        f"{k}: {'str' if v in ('text', 'wysiwyg') else v}" 
        for k, v in obj_attrs.items()]
    return '\n    '.join(model_attrs)


def __create_form_attrs(attributes: dict, helpers_path: str) -> str:
    """Create form field HTML for the specified attributes.  
    
    Args:
        attributes (dict): A dictionary of form field attributes, 
            where the keys are the field names and the values are the field types.   
        helpers_path (str): The path to the directory containing form field helper templates.  
        
    Returns:
        str: The HTML for the form fields.
    """
    helpers_path = Path(__file__).parent.resolve() / helpers_path
    form_fields = list()
    for f, t in attributes.items():
        field_helper = helpers_path / f'{t}.html'
        if not field_helper.exists():
            field_helper = helpers_path / 'str.html'
        with open(field_helper, 'r') as o:
            content = o.read()
        format_attrs = {'f': f, 'F': f.title()}
        form_fields.append(__format(content, format_attrs))
    return '\n'.join(form_fields)

def __add_wysiwyg_meta(take_action: bool):
    if take_action:
        with open(Path(__file__).parent.resolve() 
                  / 'templates/scaffold_helpers/trix_meta_content.html', 
                  'r') as o:
            return o.read()


def __get_import_placement(main_py: list) -> int:
    """Get the index at which to insert an import statement in the main Python script.  
    
    Args:
        main_py (list): A list of lines in the main Python script.  
        
    Returns:
        int: The index at which to insert an import statement.  
    """
    imp_placement = 0
    for i, line in enumerate(main_py):
        if '_router' in line and 'import' in line:
            imp_placement = i + 1
    return imp_placement


"""
GENERATORS
"""


def build_base(p: str, project_name: str, style: str) -> None:
    """Builds a base for the new project.  

    Currently supports two project bases: BOOTSTRAP and MKDOCS.  
    See templates/core/ or templates/mkdocs_core/ to see the files 
    which will be generated through the `__builder` method

    Args:
        p (str): the current working directory  
        project_name (str): name of the new project  
        style (str): style to build in.  
    """
    makedirs(p)
    content_attrs = {
        'Obj': project_name.title(),
        'project_key': config('project_key') or ''}

    gen_path = 'core'
    if style == 'MKDOCS':
        gen_path = 'mkdocs_core'

    __builder(
        project_path=p, 
        generator_path=f'templates/{gen_path}', 
        format_attrs=content_attrs)


def gen_scaffold(p: str, obj: str, attributes: list) -> None:
    """Generates Model, View, and Controller for a new Object.  

    Args:
        p (str): the current working directory  
        obj (str): the name of the new object to be created  
        attributes (list): the attributes of the new object
    """
    obj_attrs = __get_object_attrs(attributes)
    format_attrs = {
        'proj': Path.cwd().name,
        'obj': obj, 
        'Obj': obj.title(), 
        'model_attrs': __prepare_model_attrs(obj_attrs), 
        'form_attrs': __create_form_attrs(
            attributes=obj_attrs, 
            helpers_path='templates/scaffold_helpers'),
        'wysiwyg': __add_wysiwyg_meta('wysiwyg' in obj_attrs.values())}
    __builder(
        project_path=p, 
        generator_path='templates/scaffold', 
        format_attrs=format_attrs,
        scaffold_obj_name=obj)
    update(add_to_main(p, obj))

    

    with open(Path(__file__).parent.resolve() / 'templates/scaffold_helpers/scaffold_navlink.html', 'r') as o:
        scaffold_link = __format(o.read(), format_attrs)

    update(add_to_navlinks(
        p,
        content=scaffold_link))


def gen_authlib(p: str) -> None:
    """Generates a minimal authorization framework.

    Based on AuthLib library, the generate authorization framework
    allows users to login with Google. See AuthLib library for 
    other potential Open ID providers.  

    Args:
        p (str): the current working directory  
    """   
    obj = 'user'
    format_attrs = {
        'proj': Path.cwd().name,
        'obj': obj, 
        'Obj': obj.title() }

    __builder(
        project_path=p, 
        generator_path='templates/authlib_users', 
        format_attrs=format_attrs,
        scaffold_obj_name=obj)
    update(add_to_main(
        p, 
        obj, 
        extra_imports=[
            'from starlette.middleware.sessions import SessionMiddleware\n',
            'from os import environ\n'],
        extra_includes=[
            'app.add_middleware(SessionMiddleware, secret_key=environ.get("APP_SECRET"))\n']))
    update(add_to_requirements(
        p,
        reqs=['itsdangerous', 'httpx', 'Authlib', 'pyjwt', 'passlib[bcrypt]']))
    update(add_to_env(
        p,
        APP_SECRET='D' + token_urlsafe(16),
        GOOGLE_CLIENT_ID="",
        GOOGLE_CLIENT_SECRET=""))
    
    with open(Path(__file__).parent.resolve() / 'templates/scaffold_helpers/user_login.html', 'r') as o:
        user_links = o.read()

    update(add_to_navlinks(
        p,
        content=user_links))


"""
UPDATERS
"""
@dataclass 
class File:
    filepath: str 
    content: str
    mode: str = 'w'


def update(file: File):
    with open(file.filepath, file.mode) as m:
        m.write(file.content)

def add_to_main(
    p: str, 
    name: str,
    extra_imports: list or None = None, 
    extra_includes: list or None = None
) -> File:
    """Adds provided data to the main router with the scaffold router imports.  

    When a new scaffolded MVC is added to the project, the router for
    it should be added to main.py. This method will import the router 
    and include the router in the main `app`, along with the appropriate
    prefix. Extra imports and app additions may be appended as well.  

    Args:
        p (str): The path to the directory containing the main file.  
        name (str): name of the router to include.  
        extra_imports (listorNone, optional): additional imports to include. Defaults to None.  
        extra_includes (listorNone, optional): additional lines of code to include. Defaults to None.
    
    Return:
        File: the updated file and metadata
    """
    main_file = Path(p) / 'main.py'
    with open(main_file, 'r') as m:
        main = m.readlines()
    imprt = [f'from {name}.router import {name}_router\n']
    incld = [f'app.include_router({name}_router, tags=["{name}"], prefix="/{name}")\n']
    imprt += extra_imports or list()
    incld += extra_includes or list()
    main.insert(
        __get_import_placement(main), 
        ''.join(imprt))
    main.append(''.join(incld))
    return File(
        filepath=main_file, 
        content=''.join(main), 
        mode='w')


def add_to_requirements(p: str, reqs: list) -> File:
    """Adds provided data to the requirements file at the specified path with the provided requirements.  

    Args:
        p (str): The path to the directory containing the requirements file.  
        reqs (list): A list of requirements to be added to the requirements 
        
    Return:
        File: the updated file and metadata
    """
    req_file = Path(p) / 'requirements.txt'
    with open(req_file, 'r') as m:
        file_reqs = m.readlines()
    
    file_reqs.insert(0, '\n'.join(reqs) + '\n')
    return File(
        filepath=req_file, 
        content=''.join(file_reqs), 
        mode='w')


def add_to_env(p: str, **kwargs) -> File:
    """Adds provided data to the env file at the specified path with the provided variables.  

    Args:
        p (str): The path to the directory containing the env file.  
        kwargs: key and value for each entry 
        
    Return:
        File: the updated file and metadata
    """
    env_file = Path(p) / '.env'
    with open(env_file, 'r') as e:
        file_env = e.readlines()
    file_env += [
        f"{k}={v}"
        for k, v in kwargs.items()]
    return File(
        filepath=env_file, 
        content='\n'.join(file_env), 
        mode='w')


def add_to_navlinks(p: str, content: str):
    navlink_file = Path(p) / 'static_pages' / 'templates' / '_navlinks.html'
    with open(navlink_file, 'r') as e:
        file_navlink = e.read()
    return File(
        filepath=navlink_file,
        content=file_navlink + '\n\n' + content,
        mode='w')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['detamvc',
 'detamvc.extras',
 'detamvc.templates.authlib_users',
 'detamvc.templates.core',
 'detamvc.templates.core.static_pages',
 'detamvc.templates.mkdocs_core',
 'detamvc.templates.scaffold']

package_data = \
{'': ['*'],
 'detamvc': ['templates/authlib_users/templates/*',
             'templates/core/static_pages/static/css/*',
             'templates/core/static_pages/static/img/*',
             'templates/core/static_pages/static/js/*',
             'templates/core/static_pages/templates/*',
             'templates/mkdocs_core/docs/*',
             'templates/mkdocs_core/docs/assets/css/*',
             'templates/mkdocs_core/docs/assets/img/*',
             'templates/mkdocs_core/docs/tutorial/*',
             'templates/scaffold/templates/*',
             'templates/scaffold_helpers/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0',
 'authlib>=1.2.0,<2.0.0',
 'deta>=1.1.0,<2.0.0',
 'fastapi>=0.78.0,<0.79.0',
 'httpx>=0.23.1,<0.24.0',
 'itsdangerous>=2.1.2,<3.0.0',
 'mkdocs-material>=8.5.7,<9.0.0',
 'odetam>=1.2.0,<2.0.0',
 'passlib[bcrypt]>=1.7.4,<2.0.0',
 'pyjwt>=2.6.0,<3.0.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'python-multipart>=0.0.5,<0.0.6',
 'typer[all]>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['detamvc = detamvc.main:app']}

setup_kwargs = {
    'name': 'detamvc',
    'version': '0.4.3',
    'description': '',
    'long_description': '![DetaMVC](https://detamvc.deta.dev/assets/images/detamvc.png)    \n\n\n**Documentation:** [https://detamvc.deta.dev](https://detamvc.deta.dev)  \n**Source Code:** [https://github.com/pyn-sol/detaMVC](https://github.com/pyn-sol/detaMVC)  \n\n\nDetaMVC is a framework for rapidly developing and deploying web applications using:\n- [FastAPI](https://fastapi.tiangolo.com/)\n- [Jinja2](https://fastapi.tiangolo.com/advanced/templates/?h=jinja2)\n- [Deta](https://docs.deta.sh/docs/home)  \n\n\n## Installation\n```\npip install detamvc\n```\n\nOther Requirements:\n- A Deta Account. If you do not have one, go to [Deta](https://www.deta.sh/) and click \'Join Deta\'\n- The [Deta CLI](https://docs.deta.sh/docs/cli/install)\n\n\n## Basics\nIf you are familiar with Ruby on Rails, the commands are very similar for creating an application. \n\n```\ndetamvc new project\n\ncd project\n\ndetamvc scaffold item name:str description:text price:float quantity:int available:bool\n```\n\nBefore running your project, be sure to set your PROJECT_KEY for Deta. You can get this from your Deta dashboard under \'settings\'.\n\n\n```\necho DETA_PROJECT_KEY="#######_#############" > .env\n```\n\nOr, save yourself the hassle and set your development project key using the command. Hint: Do this _before_ creating a new project.\n\n```\ndetamvc set-project-key #######_#################\n```\n\n## Run a Server Manually\n\nThis assumes you have uvicorn installed. You can run with other servers as you wish - just set up like you would for a normal [FastAPI](https://fastapi.tiangolo.com/deployment/manually/ "Run a Server Manually - Uvicorn") application.\n```\ndetamvc s\n```\nor\n```\nuvicorn main:app --reload\n```\n\n## Deploy on Deta\nNow you can deploy this on Deta!\nBefore running the following, you will need to install the [Deta CLI](https://docs.deta.sh/docs/cli/install)\n```\ndeta new --project default\n```',
    'author': 'MBeebe',
    'author_email': 'grow.food.everywhere@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from fastapi import FastAPI, Response
from fastapi.staticfiles import StaticFiles
from os.path import isfile
from mimetypes import guess_type


app = FastAPI(docs_url=None, redoc_url=None)
app.mount("/assets", StaticFiles(directory="site/assets"), name="assets")
app.mount("/search", StaticFiles(directory="site/search"), name="search")



@app.get('/{filename:path}')
def mkdocsite(filename: str):
    fn = './site/' + filename + 'index.html'
    if not isfile(fn):
        return Response(status_code=404)

    with open(fn) as f:
        content = f.read()

    content_type, _ = guess_type(fn)
    return Response(content, media_type=content_type)
